@@ -1,6 +1,9 @@
 from __future__ import annotations
 
+import warnings
+
 import jijmodeling as jm
+from jijmodeling.deprecation.deprecation import JijFutureWarning
 
 
 def travelling_salesman_with_time_windows():
@@ -8,23 +11,25 @@ def travelling_salesman_with_time_windows():
     problem = jm.Problem("travelling-salesman-with-time-windows")
 
     # 距離行列
-    dist = jm.Placeholder("d", dim=2)  # 距離行列
+    dist = jm.Placeholder("d", ndim=2)  # 距離行列
     N = jm.Placeholder("N")
-    e = jm.Placeholder("e", shape=(N,))  # ready time
-    l = jm.Placeholder("l", shape=(N,))  # due time
-    x = jm.Binary("x", shape=(N, N))
-    t = jm.Integer("t", shape=(N,), lower=e, upper=l)
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", category=JijFutureWarning)
+        e = jm.Placeholder("e", shape=(N,))  # ready time
+        l = jm.Placeholder("l", shape=(N,))  # due time
+    x = jm.BinaryVar("x", shape=(N, N))
+    t = jm.IntegerVar("t", shape=(N,), lower_bound=e, upper_bound=l)
 
-    i = jm.Element("i", N)
-    j = jm.Element("j", N)
+    i = jm.Element("i", belong_to=N)
+    j = jm.Element("j", belong_to=N)
 
     # Objevtive Function: 距離の最小化
     sum_list = [i, (j, j != i)]
-    obj = jm.Sum(sum_list, dist[i, j] * x[i, j])
+    obj = jm.sum(sum_list, dist[i, j] * x[i, j])
     problem += obj
 
     # Const1: 都市iから出る辺は1つ
-    term1 = jm.Sum((j, j != i), x[i, j])
+    term1 = jm.sum((j, j != i), x[i, j])
     problem += jm.Constraint(
         "onehot-constraint1",
         term1 == 1,
@@ -34,7 +39,7 @@ def travelling_salesman_with_time_windows():
     )
 
     # Const2: 都市iに入る辺は1つ
-    term2 = jm.Sum((j, j != i), x[j, i])
+    term2 = jm.sum((j, j != i), x[j, i])
     problem += jm.Constraint(
         "onehot-constraint2",
         term2 == 1,
